@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-
+import mmap
 from collections import Callable
 import os
 from typing import Optional
@@ -18,17 +18,15 @@ from dataloader.util.misc import get_offset
 class RawDataset:
     data_kinds = {DataKind.FILE, DataKind.MMAP_FILE, DataKind.MEM_SEQ}
 
-    def __init__(self, data, kind=DataKind.MEM_SEQ, transform: Optional[Callable] = None, filename=None):
+    def __init__(self, data, kind=DataKind.MEM_SEQ, transform: Optional[Callable] = None):
         """
 
         Args:
             data:
               - list of things: if kind=DataKind.MEM_SEQ
-              - filename: if kind=DataKind.FILE
-              - mmap: if kind=DataKind.MMAP_FILE
+              - filename: if kind=DataKind.FILE or kind=DataKind.MMAP_FILE
             kind: @see DataKind
             transform: func for mapping raw data into feature
-            filename: only valid if kind=DataKind.MMAP_FILE
 
         >>> import os, mmap
         >>> fp = open('filename', 'rb', os.O_RDONLY)
@@ -53,23 +51,22 @@ class RawDataset:
 
             self.data = data
             self.n_data = len(data)
-        elif self.kind == DataKind.FILE:
+        elif self.kind == DataKind.FILE or self.kind == DataKind.MMAP_FILE:
             if not os.path.exists(data):
                 raise ValueError(f'filename does not exist: {data}')
             self.filename = data
 
             self.offset, self.n_data = get_offset(self.filename)
-            self.fd = open(data, 'rb', buffering=0)
-            self.lock = multiprocessing.Lock()
-            logger.warning('with multiprocessing.Lock lead to poor speed')
-        elif self.kind == DataKind.MMAP_FILE:
-            if not os.path.exists(filename):
-                raise ValueError(f'filename does not exist: {filename}')
-            self.filename = filename
 
-            self.offset, self.n_data = get_offset(self.filename)
-            self.data = data  # mmap
-            logger.warning('with mmap lead to huge memory cost')
+            if self.kind == DataKind.FILE:
+                self.fd = open(data, 'rb', buffering=0)
+                self.lock = multiprocessing.Lock()
+                logger.warning('with multiprocessing.Lock lead to poor speed')
+            elif self.kind == DataKind.MMAP_FILE:
+                fp = os.open(self.filename, os.O_RDONLY)
+                self.data = mmap.mmap(fp, 0, access=mmap.ACCESS_READ)
+
+                logger.warning('with mmap lead to huge memory cost')
 
         self.transform = transform
 
